# 中文版说明
本工具用于判定正则表达式*r*的弱确定性和强确定性，由陈子轩修改自牟笑莹的Udtool。

程序的的主函数为Expression1.java，输入为一个正则表达式*r*，输出为 (1) *r*是否为弱确定性的 (2) *r*是否为强确定性的

输入格式为：连接用`,`表示，counting支持`[m,n]`、`[m]`和`[m,]`表示

例如*r* = `a[2,],a`，输出为
```
------------------------------------------
--The simplified expression is 1[2,2147483647],1.
--Diagnosis：the subexpression (a[2,2147483647],a) is nondeterministic, 
since there are two marking symbols with the same underlying symbol{a} 
between the followlast set of (a[2,2147483647]) and the first set of (a) . 
    !The expression is weakly deterministic.
------------------------------------------
--The simplified expression is1[2,2147483647],1.
--Diagnosis：the subexpression (a[2,2147483647],a) is nondeterministic, 
since there are two marking symbols with the same underlying symbol{a} 
between the followlast set of (a[2,2147483647]) and the first set of (a) . 
    !The expression is deterministic.
The second step to check wSNF:
    ! The expression is in wSNF form.
    So the expression is deterministic for SDRE.
***********************************************************
```

解释如下，首先counting`[2,]`会替换为`[2,2147483647]`，然后simplified expression是牟对正则表达式*r*的化简。
下面分为两部分：第一部分是判定*r*是否满足弱确定性；第二部分是若*r*满足弱确定性，进一步判断是否满足强确定性。在每一部中都给出原因。
可以看到在本例中，`a[2,],a`不是弱确定性的，因为符号`a`同时出现在`followlast(a[2,])`和`first(a)`中。进而也不满足强确定性。

再如*r* = `(a*)*`，输出为
```
------------------------------------------
--The simplified expression is (1[0,2147483647])[0,2147483647].
    !The expression is weakly deterministic.
------------------------------------------
--The simplified expression is(1[0,2147483647])[0,2147483647].
    !The expression is deterministic.
The second step to check wSNF:
    !!! The expression is not in wSNF form.
    So the expression is nondeterministic for SDRE!
***********************************************************
```

可见该表达式是弱确定性的，但不满足wSNF形式，所以不是强确定性的。

最后如*r* = `a*,b+`，输出为

```
------------------------------------------
--The simplified expression is 1[0,2147483647],2[1,2147483647].
    !The expression is weakly deterministic.
------------------------------------------
--The simplified expression is1[0,2147483647],2[1,2147483647].
    !The expression is deterministic.
The second step to check wSNF:
    ! The expression is in wSNF form.
    So the expression is deterministic for SDRE.
***********************************************************
```

可见该表达式即是强确定性的也是弱确定性的。


# 英文版说明

This tool is used to judge the weak deterministic and strong deterministic of regular expression *r*, which is modified by Chen Zixuan from Mou Xiaoying's Udtool.

The main function of the program is Expression1.java, the input is a regular expression *r*, and the output is (1) *r* whether it is weakly deterministic (2) *r* whether it is strongly deterministic

The input format is `,` for concatenation, and `[m,n]`, `[m]` and `[m,]` for counting.


For example, *r* = `a[2,],a`, the output is
```
------------------------------------------
--The simplified expression is 1[2,2147483647],1.
--Diagnosis：the subexpression (a[2,2147483647],a) is nondeterministic, 
since there are two marking symbols with the same underlying symbol{a} 
between the followlast set of (a[2,2147483647]) and the first set of (a) . 
    !The expression is weakly deterministic.
------------------------------------------
--The simplified expression is1[2,2147483647],1.
--Diagnosis：the subexpression (a[2,2147483647],a) is nondeterministic, 
since there are two marking symbols with the same underlying symbol{a} 
between the followlast set of (a[2,2147483647]) and the first set of (a) . 
    !The expression is deterministic.
The second step to check wSNF:
    ! The expression is in wSNF form.
    So the expression is deterministic for SDRE.
***********************************************************
```

The explanation is as follows. First, counting `[2,]` will be replaced by `[2,2147483647]`, and then simplified expression is Mou's simplification of regular expression *r*.
The following contains two parts: the first part is to determine whether *r* is weakly deterministic; The second part is to further judge whether it is strong deterministic if *r* is weak determinism. The reasons are given in each part.
It can be seen that in this example, `a[2,],a` is not weakly deterministic, because the symbol `a` appears in both `followlast(a[2,])` and `first(a)`. Furthermore, it does not satisfy strong determinism.

For another example, *r* = `(a*)*`, the output is
```
------------------------------------------
--The simplified expression is (1[0,2147483647])[0,2147483647].
    !The expression is weakly deterministic.
------------------------------------------
--The simplified expression is(1[0,2147483647])[0,2147483647].
    !The expression is deterministic.
The second step to check wSNF:
    !!! The expression is not in wSNF form.
    So the expression is nondeterministic for SDRE!
***********************************************************
```


It can be seen that the expression is weakly deterministic, but does not satisfy the wSNF form, so it is not strongly deterministic.

Finally, for *r* = `a*,b+`, the output is

```
------------------------------------------
--The simplified expression is 1[0,2147483647],2[1,2147483647].
    !The expression is weakly deterministic.
------------------------------------------
--The simplified expression is1[0,2147483647],2[1,2147483647].
    !The expression is deterministic.
The second step to check wSNF:
    ! The expression is in wSNF form.
    So the expression is deterministic for SDRE.
***********************************************************
```

It can be seen that the expression is both strongly deterministic and weakly deterministic.

